@@ -47,7 +47,8 @@ CtMainWin::CtMainWin(CtConfig*        pCtConfig,
    _rGtkCssProvider(rGtkCssProvider),
    _pGsvLanguageManager(pGsvLanguageManager),
    _pGsvStyleSchemeManager(pGsvStyleSchemeManager),
-   _ctTextview(this)
+   _ctTextview(this),
+   _ctStateMachine(this)
 {
     set_icon(_pGtkIconTheme->load_icon(CtConst::APP_NAME, 48));
     _scrolledwindowTree.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
@@ -65,6 +66,7 @@ CtMainWin::CtMainWin(CtConfig*        pCtConfig,
         _hPaned.add1(_scrolledwindowTree);
         _hPaned.add2(_vboxText);
     }
+    _hPaned.property_wide_handle() = true;
 
     _pMenuBar = pCtMenu->build_menubar();
     _pMenuBar->set_name("MenuBar");
@@ -91,15 +93,8 @@ CtMainWin::CtMainWin(CtConfig*        pCtConfig,
     _ctTextview.signal_size_allocate().connect(sigc::mem_fun(*this, &CtMainWin::_on_textview_size_allocate));
     _ctTextview.signal_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_textview_event));
     _ctTextview.signal_event_after().connect(sigc::mem_fun(*this, &CtMainWin::_on_textview_event_after));
-    _ctTextview.signal_scroll_event().connect([this](GdkEventScroll* event)
-    {
-        if (event->state & GDK_CONTROL_MASK and (event->direction == GDK_SCROLL_UP or event->direction == GDK_SCROLL_DOWN))
-        {
-            _ctTextview.zoom_text(event->direction == GDK_SCROLL_UP);
-            return true;
-        }
-        return false;
-    });
+    _ctTextview.signal_scroll_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_textview_scroll_event));
+
     _uCtPairCodeboxMainWin.reset(new CtPairCodeboxMainWin{nullptr, this});
     g_signal_connect(G_OBJECT(_ctTextview.gobj()), "cut-clipboard", G_CALLBACK(CtClipboard::on_cut_clipboard), _uCtPairCodeboxMainWin.get());
     g_signal_connect(G_OBJECT(_ctTextview.gobj()), "copy-clipboard", G_CALLBACK(CtClipboard::on_copy_clipboard), _uCtPairCodeboxMainWin.get());
@@ -141,7 +136,15 @@ Glib::RefPtr<Gsv::Buffer> CtMainWin::get_new_text_buffer(const std::string& synt
     Glib::RefPtr<Gsv::Buffer> rRetTextBuffer;
     rRetTextBuffer = Gsv::Buffer::create(_rGtkTextTagTable);
     rRetTextBuffer->set_max_undo_levels(_pCtConfig->limitUndoableSteps);
-    if (CtConst::RICH_TEXT_ID != syntax)
+    if (CtConst::RICH_TEXT_ID == syntax)
+    {
+        // dark theme
+        if (get_ct_config()->rtDefFg == CtConst::RICH_TEXT_DARK_FG && get_ct_config()->rtDefBg == CtConst::RICH_TEXT_DARK_BG)
+            rRetTextBuffer->set_style_scheme(_pGsvStyleSchemeManager->get_scheme("cobalt"));
+        else
+            rRetTextBuffer->set_style_scheme(_pGsvStyleSchemeManager->get_scheme("classic"));
+    }
+    else
     {
         rRetTextBuffer->set_style_scheme(_pGsvStyleSchemeManager->get_scheme(_pCtConfig->styleSchemeId));
         if (CtConst::PLAIN_TEXT_ID == syntax)
@@ -381,33 +384,35 @@ bool CtMainWin::apply_tag_try_automatic_bounds(Glib::RefPtr<Gtk::TextBuffer> tex
     return false;
 }
 
-std::string CtMainWin::get_font_css(const std::string& fontStr)
+// Try to select the full paragraph
+bool CtMainWin::apply_tag_try_automatic_bounds_triple_click(Glib::RefPtr<Gtk::TextBuffer> text_buffer, Gtk::TextIter iter_start)
 {
-    g_autofree gchar* pFontCss = g_strdup_printf(
-        "textview text {"
-        "    font-family: %s;"
-        "    font-size: %spx;"
-        "}", CtFontUtil::get_font_family(fontStr).c_str(), CtFontUtil::get_font_size_str(fontStr).c_str());
-    std::string fontCss(pFontCss);
-    return fontCss;
-}
+    Gtk::TextIter iter_end = iter_start;
+    iter_end.forward_to_line_end();
 
-const std::string& CtMainWin::get_font_for_syntax_highlighting(const std::string& syntaxHighlighting)
-{
-    if (0 == syntaxHighlighting.compare(CtConst::RICH_TEXT_ID))
+    iter_end.forward_char();
+    auto next_char = iter_end.get_char();
+    while (next_char != '\n' && next_char != ' ')
     {
-        return _pCtConfig->rtFont;
+        // forward to the end of the line, if the next char
+        // is not a new line or space then repeat
+        iter_end.forward_to_line_end();
+        if (!iter_end.forward_char()) break;
+        next_char = iter_end.get_char();
     }
-    if (0 == syntaxHighlighting.compare(CtConst::PLAIN_TEXT_ID))
-    {
-        return _pCtConfig->ptFont;
-    }
-    return _pCtConfig->codeFont;
-}
 
-std::string CtMainWin::get_font_css_for_syntax_highlighting(const std::string& syntaxHighlighting)
-{
-    return get_font_css(get_font_for_syntax_highlighting(syntaxHighlighting));
+    // reverse to beginning of line to check for space indicating line
+    // selected is the first line of a paragraph
+    iter_start.backward_chars(iter_start.get_visible_line_offset());
+    // reverse until either a new line or a space is found
+    while (iter_start.get_char() != '\n' && iter_start.get_char() != ' ')
+        if (!iter_start.backward_line()) break;
+
+    if (iter_start.get_char() == '\n')
+        iter_start.forward_chars(1);
+
+    text_buffer->move_mark(text_buffer->get_insert(), iter_start);
+    text_buffer->move_mark(text_buffer->get_selection_bound(), iter_end);
 }
 
 void CtMainWin::_reset_CtTreestore_CtTreeview()
@@ -426,6 +431,7 @@ void CtMainWin::_reset_CtTreestore_CtTreeview()
     _uCtTreeview->signal_cursor_changed().connect(sigc::mem_fun(*this, &CtMainWin::_on_treeview_cursor_changed));
     _uCtTreeview->signal_button_release_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_treeview_button_release_event));
     _uCtTreeview->signal_key_press_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_treeview_key_press_event), false);
+    _uCtTreeview->signal_scroll_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_treeview_scroll_event));
     _uCtTreeview->signal_popup_menu().connect(sigc::mem_fun(*this, &CtMainWin::_on_treeview_popup_menu));
 
     _uCtTreeview->get_style_context()->add_class("ct_node_view");
@@ -470,7 +476,7 @@ void CtMainWin::config_update_data_from_curr_status()
 }
 
 void CtMainWin::configure_theme()
-{
+{ 
     auto font_to_string = [](Pango::FontDescription font)
     {
         return " { font-family: " + font.get_family() +
@@ -483,13 +489,6 @@ void CtMainWin::configure_theme()
     std::string codeFont = font_to_string(Pango::FontDescription(_pCtConfig->codeFont));
     std::string treeFont = font_to_string(Pango::FontDescription(_pCtConfig->treeFont));
 
-    auto screen = get_screen();
-    if (_css_provider_theme_font)
-    {
-        Gtk::StyleContext::remove_provider_for_screen(screen, _css_provider_theme_font);
-    }
-    _css_provider_theme_font = Gtk::CssProvider::create();
-
     std::string font_css;
     font_css += ".ct_textview.rich-text" + rtFont;
     font_css += ".ct_textview.plain-text" + plFont;
@@ -499,26 +498,19 @@ void CtMainWin::configure_theme()
     font_css += ".codebox.code" + codeFont;
     font_css += ".ct_node_view" + treeFont;
 
-    _css_provider_theme_font->load_from_data(font_css);
-    get_style_context()->add_provider_for_screen(screen, _css_provider_theme_font, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-
-#ifdef THEME_FIXES_REQUIRED
-    // can do more complicated things than just changing colors
-    if (_css_provider_theme)
-    {
-        Gtk::StyleContext::remove_provider_for_screen(screen, _css_provider_theme);
-    }
-    _css_provider_theme = Gtk::CssProvider::create();
     std::string theme_css;
-    // todo: rich text fix light selected line with dark theme
-    theme_css += ".ct_textview.rich-text > text { color: " + _pCtConfig->rtDefFg + "; background-color: " + _pCtConfig->rtDefBg + "; } ";
-    // todo: tree selected node highlight no longer working
     theme_css += ".ct_node_view { color: " + _pCtConfig->ttDefFg + "; background-color: " + _pCtConfig->ttDefBg + "; } ";
+    theme_css += ".ct_node_view:selected { background: #5294e2;  } ";
     theme_css += ".ct_header { background-color: " + _pCtConfig->ttDefBg + "; } ";
 
+    if (!_css_provider_theme)
+    {
+        Gtk::StyleContext::remove_provider_for_screen(get_screen(), _css_provider_theme);
+    }
+    _css_provider_theme = Gtk::CssProvider::create();
+    _css_provider_theme->load_from_data(font_css);
     _css_provider_theme->load_from_data(theme_css);
-    get_style_context()->add_provider_for_screen(screen, _css_provider_theme, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-#endif // THEME_FIXES_REQUIRED
+    get_style_context()->add_provider_for_screen(get_screen(), _css_provider_theme, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 }
 
 Gtk::HBox& CtMainWin::_init_status_bar()
@@ -527,7 +519,10 @@ Gtk::HBox& CtMainWin::_init_status_bar()
     _ctStatusBar.frame.set_shadow_type(Gtk::SHADOW_NONE);
     _ctStatusBar.frame.set_border_width(1);
     _ctStatusBar.frame.add(_ctStatusBar.progressBar);
-    _ctStatusBar.stopButton.set_image_from_icon_name("gtk-stop", Gtk::ICON_SIZE_MENU);
+    _ctStatusBar.stopButton.set_image_from_icon_name("stop", Gtk::ICON_SIZE_MENU);
+    _ctStatusBar.statusBar.set_margin_top(0);
+    _ctStatusBar.statusBar.set_margin_bottom(0);
+    _ctStatusBar.hbox.set_border_width(0);
     _ctStatusBar.hbox.pack_start(_ctStatusBar.statusBar, true, true);
     _ctStatusBar.hbox.pack_start(_ctStatusBar.frame, false, true);
     _ctStatusBar.hbox.pack_start(_ctStatusBar.stopButton, false, true);
@@ -680,6 +675,16 @@ void CtMainWin::_ensure_curr_doc_in_recent_docs()
     }
 }
 
+void CtMainWin::_zoom_tree(bool is_increase)
+{
+    Glib::RefPtr<Gtk::StyleContext> context = _uCtTreeview->get_style_context();
+    Pango::FontDescription description = context->get_font(context->get_state());
+    auto size = description.get_size() / Pango::SCALE + (is_increase ? 1 : -1);
+    if (size < 6) size = 6;
+    description.set_size(size * Pango::SCALE);
+    _uCtTreeview->override_font(description);
+}
+
 bool CtMainWin::filepath_open(const std::string& filepath, const bool force_reset)
 {
     _ensure_curr_doc_in_recent_docs();
@@ -708,8 +713,7 @@ bool CtMainWin::reset(const bool force_reset)
     _set_new_curr_doc(Glib::RefPtr<Gio::File>{nullptr}, "");
 
     update_window_save_not_needed();
-    //todo
-    //self.state_machine.reset()
+    get_state_machine().reset();
     return true;
 }
 
@@ -826,6 +830,7 @@ bool CtMainWin::read_nodes_from_gio_file(const Glib::RefPtr<Gio::File>& r_file,
                                                        &_ctTextview,
                                                        iterDocsRestore->second.node_path,
                                                        iterDocsRestore->second.cursor_pos);
+            _ctTextview.grab_focus();
         }
     }
     return retOk;
@@ -964,16 +969,58 @@ void CtMainWin::update_window_save_needed(const CtSaveNeededUpdType update_type,
             std::vector<gint64> rm_node_ids = treeIter.get_children_node_ids();
             rm_node_ids.push_back(top_node_id);
             _uCtTreestore->pending_rm_db_nodes(rm_node_ids);
-            //todo
-            //self.state_machine.delete_states(node_id) 
+            for (auto node_id: rm_node_ids)
+                get_state_machine().delete_states(node_id);
         } break;
         case CtSaveNeededUpdType::book:
         {
             _uCtTreestore->pending_edit_db_bookmarks();
         } break;
     }
-    //todo
-    //if new_state_machine and tree_iter: self.state_machine.update_state()
+    if (new_machine_state && treeIter)
+        get_state_machine().update_state(treeIter);
+}
+
+// Load Text Buffer from State Machine
+void CtMainWin::load_buffer_from_state(std::shared_ptr<CtNodeState> state, CtTreeIter tree_iter)
+{
+    // todo:
+    // spell_check_restore = self.enable_spell_check
+    // self.toggle_ena_dis_spellcheck()
+    bool user_active_restore = user_active();
+    user_active() = false;
+
+    auto text_buffer = tree_iter.get_node_text_buffer();
+    Glib::RefPtr<Gsv::Buffer> gsv_buffer = Glib::RefPtr<Gsv::Buffer>::cast_dynamic(text_buffer);
+
+    text_buffer->begin_not_undoable_action();
+
+    text_buffer->erase(text_buffer->begin(), text_buffer->end());
+    std::list<CtAnchoredWidget*> widgets;
+    for (xmlpp::Node* text_node: state->buffer_xml.get_root_node()->get_children())
+    {
+        CtXmlRead(this).get_text_buffer_slot(gsv_buffer, nullptr, widgets, text_node);
+    }
+    tree_iter.remove_all_embedded_widgets();
+    // CtXmlRead(this).get_text_buffer_slot didn't fill widgets, they are kept separately
+    for (auto widget: state->widgets)
+        widgets.push_back(widget->clone());
+    for (auto widget: widgets)
+        widget->insertInTextBuffer(gsv_buffer);
+    curr_tree_store().addAnchoredWidgets(tree_iter, widgets, &get_text_view());
+
+    text_buffer->end_not_undoable_action();
+    text_buffer->set_modified(false);
+
+    get_text_view().set_buffer(text_buffer);
+    text_buffer->place_cursor(text_buffer->get_iter_at_offset(state->cursor_pos));
+    get_text_view().scroll_to(text_buffer->get_insert(), CtTextView::TEXT_SCROLL_MARGIN);
+
+    user_active() = user_active_restore;
+    // todo:
+    // if not given_tree_iter:
+    //    if spell_check_restore: self.toggle_ena_dis_spellcheck()
+    update_window_save_needed(CtSaveNeededUpdType::nbuf, false, &tree_iter);
 }
 
 void CtMainWin::_on_treeview_cursor_changed()
@@ -985,7 +1032,7 @@ void CtMainWin::_on_treeview_cursor_changed()
         {
             _fileSaveNeeded = true;
             rTextBuffer->set_modified(false);
-            //self.state_machine.update_state()
+            get_state_machine().update_state(_prevTreeIter);
         }
     }
     CtTreeIter treeIter = curr_tree_iter();
@@ -996,6 +1043,7 @@ void CtMainWin::_on_treeview_cursor_changed()
     window_header_update_lock_icon(treeIter.get_node_read_only());
     window_header_update_bookmark_icon(false);
     update_selected_node_statusbar_info();
+    get_state_machine().node_selected_changed(treeIter.get_node_id());
 
     _prevTreeIter = treeIter;
 }
@@ -1012,7 +1060,6 @@ bool CtMainWin::_on_treeview_button_release_event(GdkEventButton* event)
 
 bool CtMainWin::_on_window_key_press_event(GdkEventKey* event)
 {
-    // todo:
     if (event->state & GDK_CONTROL_MASK) {
         if (event->keyval == GDK_KEY_Tab) {
             _pCtActions->toggle_tree_text();
@@ -1024,29 +1071,107 @@ bool CtMainWin::_on_window_key_press_event(GdkEventKey* event)
 
 bool CtMainWin::_on_treeview_key_press_event(GdkEventKey* event)
 {
-    // todo:
     if (not curr_tree_iter()) return false;
     if (event->state & GDK_SHIFT_MASK) {
-        if (event->keyval == GDK_KEY_Up) {
+        if (event->state & GDK_CONTROL_MASK && event->keyval == GDK_KEY_Right) {
+            _pCtActions->node_change_father();
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Up) {
             _pCtActions->node_up();
             return true;
         }
-        if (event->keyval == GDK_KEY_Down) {
+        else if (event->keyval == GDK_KEY_Down) {
             _pCtActions->node_down();
             return true;
         }
+        else if (event->keyval == GDK_KEY_Left) {
+            _pCtActions->node_left();
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Right) {
+            _pCtActions->node_right();
+            return true;
+        }
     }
     else if (event->state & GDK_MOD1_MASK) {
 
     }
     else if (event->state & GDK_CONTROL_MASK) {
+        auto reduce = [](Gtk::TreeIter first, std::function<Gtk::TreeIter(Gtk::TreeIter)> operatr) -> Gtk::TreeIter
+        {
+            Gtk::TreeIter result;
+            for (auto next = operatr(first); next; next = operatr(next))
+                result = next;
+            return result;
+        };
 
+        if (event->keyval == GDK_KEY_Up) {
+            auto fist_sibling = reduce(curr_tree_iter(), [](Gtk::TreeIter iter) { return --iter;});
+            if (fist_sibling)
+                curr_tree_view().set_cursor_safe(fist_sibling);
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Down) {
+            auto last_sibling = reduce(curr_tree_iter(), [](Gtk::TreeIter iter) { return ++iter;});
+            if (last_sibling)
+                curr_tree_view().set_cursor_safe(last_sibling);
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Left) {
+            auto fist_parent = reduce(curr_tree_iter(), [](Gtk::TreeIter iter) { return iter->parent();});
+            if (fist_parent)
+                curr_tree_view().set_cursor_safe(fist_parent);
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Right) {
+            auto last_child = reduce(curr_tree_iter(), [](Gtk::TreeIter iter) { return iter->children().begin();});
+            if (last_child)
+                curr_tree_view().set_cursor_safe(last_child);
+            return true;
+        } else {
+            if (event->keyval == GDK_KEY_plus || event->keyval == GDK_KEY_KP_Add || event->keyval == GDK_KEY_equal) {
+                _zoom_tree(true);
+                return true;
+            }
+            else if (event->keyval == GDK_KEY_minus|| event->keyval == GDK_KEY_KP_Subtract) {
+                _zoom_tree(false);
+                return true;
+            }
+        }
     }
     else {
-        if (event->keyval == GDK_KEY_Tab) {
+        if (event->keyval == GDK_KEY_Left) {
+            if (_uCtTreeview->row_expanded(_uCtTreestore->get_path(curr_tree_iter())))
+                _uCtTreeview->collapse_row(_uCtTreestore->get_path(curr_tree_iter()));
+            else if (curr_tree_iter().parent())
+                curr_tree_view().set_cursor_safe(curr_tree_iter().parent());
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Right) {
+            curr_tree_view().expand_row(curr_tree_store().get_path(curr_tree_iter()), false);
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Return) {
+            auto path = curr_tree_store().get_path(curr_tree_iter());
+            if (_uCtTreeview->row_expanded(path))
+                _uCtTreeview->collapse_row(path);
+            else
+                _uCtTreeview->expand_row(path, false);
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Menu) {
+            _pCtMenu->get_popup_menu(CtMenu::POPUP_MENU_TYPE::Node)->popup(0, event->time);
+            return true;
+        }
+        else if (event->keyval == GDK_KEY_Tab) {
             _pCtActions->toggle_tree_text();
             return true;
         }
+        else if (event->keyval == GDK_KEY_Delete) {
+            _pCtActions->node_delete();
+            return true;
+        }
     }
     return false;
 }
@@ -1057,6 +1182,17 @@ bool CtMainWin::_on_treeview_popup_menu()
     return true;
 }
 
+bool CtMainWin::_on_treeview_scroll_event(GdkEventScroll* event)
+{
+    if (!(event->state & GDK_CONTROL_MASK))
+        return false;
+    if  (event->direction == GDK_SCROLL_UP || event->direction == GDK_SCROLL_DOWN)
+        _zoom_tree(event->direction == GDK_SCROLL_UP);
+    if  (event->direction == GDK_SCROLL_SMOOTH && event->delta_y != 0)
+        _zoom_tree(event->delta_y > 0);
+    return true;
+}
+
 // Extend the Default Right-Click Menu
 void CtMainWin::_on_textview_populate_popup(Gtk::Menu* menu)
 {
@@ -1066,7 +1202,7 @@ void CtMainWin::_on_textview_populate_popup(Gtk::Menu* menu)
         /*for (auto menuitem: menu->get_children())
             if (menu->)
             try:
-                if menuitem.get_image().get_property("stock") == "gtk-paste":
+                if menuitem.get_image().get_property("stock") == "edit_paste":
                     menuitem.set_sensitive(True)
             except: pass
         */
@@ -1259,6 +1395,17 @@ bool CtMainWin::_on_textview_event(GdkEvent* event)
             }
         }
     }
+    else if (event->key.state & Gdk::CONTROL_MASK)
+    {
+        if (event->key.keyval == GDK_KEY_plus || event->key.keyval == GDK_KEY_KP_Add || event->key.keyval == GDK_KEY_equal) {
+            _ctTextview.zoom_text(true);
+            return true;
+        }
+        else if (event->key.keyval == GDK_KEY_minus|| event->key.keyval == GDK_KEY_KP_Subtract) {
+            _ctTextview.zoom_text(false);
+            return true;
+        }
+    }
     return false;
 }
 
@@ -1269,12 +1416,16 @@ void CtMainWin::_on_textview_event_after(GdkEvent* event)
     {
         get_text_view().for_event_after_double_click_button1(event);
     }
+    if (event->type == GDK_3BUTTON_PRESS and event->button.button == 1)
+    {
+        if (curr_tree_iter().get_node_is_rich_text())
+            get_text_view().for_event_after_triple_click_button1(event);
+    }
     else if (event->type == GDK_BUTTON_PRESS or event->type == GDK_KEY_PRESS)
     {
-        if (curr_tree_iter() and curr_tree_iter().get_node_syntax_highlighting() == CtConst::RICH_TEXT_ID and
-            not curr_buffer()->get_modified())
+        if (curr_tree_iter() and not curr_buffer()->get_modified())
         {
-            // todo: self.state_machine.update_curr_state_cursor_pos(self.treestore[self.curr_tree_iter][3])
+            get_state_machine().update_curr_state_cursor_pos(curr_tree_iter().get_node_id());
         }
         if (event->type == GDK_BUTTON_PRESS)
         {
@@ -1297,6 +1448,17 @@ void CtMainWin::_on_textview_event_after(GdkEvent* event)
     }
 }
 
+bool CtMainWin::_on_textview_scroll_event(GdkEventScroll* event)
+{
+    if (!(event->state & GDK_CONTROL_MASK))
+        return false;
+    if  (event->direction == GDK_SCROLL_UP || event->direction == GDK_SCROLL_DOWN)
+        _ctTextview.zoom_text(event->direction == GDK_SCROLL_UP);
+    if  (event->direction == GDK_SCROLL_SMOOTH && event->delta_y != 0)
+        _ctTextview.zoom_text(event->delta_y > 0);
+    return true;
+}
+
 void CtMainWin::_title_update(const bool saveNeeded)
 {
     Glib::ustring title;
